#include <stdint.h>
//===================  USART ==================
typedef struct usart
{
  volatile unsigned int SR;    	// 0x000  Status register
  volatile unsigned int DR;    	// 0x004  Data register
  volatile unsigned int BRR;    	// 0x008  Baud rate register
  volatile unsigned int CR1;		// 0x00C  Control register 1
  volatile unsigned int CR2; 		// 0x010  Control register 2  
  volatile unsigned int CR3;		// 0x014 Control register 3
  volatile unsigned int GTPR;		// 0x018 Guard time and prescaler register
} USART ;

#define USART1	((USART *)0x40011000)	// USART1 base adress.  outer () a MUST!!!
#define USART2	((USART *)0x40004400)	// USART2 base adress
#define USART6	((USART *)0x40011400)	// USART6 base adress

//===================  I2C ==================
typedef struct i2c
{
  volatile unsigned int CR1;    	// 0x000  Control register 1
  volatile unsigned int CR2;    	// 0x004  Control register 2
  volatile unsigned int OAR1;    	// 0x008  Own address register 1
  volatile unsigned int OAR2;		// 0x00C  Own address register 2
  volatile unsigned int DR; 		// 0x010  Data register  
  volatile unsigned int SR1;		// 0x014 Status register 1
  volatile unsigned int SR2;		// 0x018 Status register 2
  volatile unsigned int CCR;		// 0x01C Clock control register
  volatile unsigned int TRISE;		// 0x020 TRISE register
  volatile unsigned int FLTR;		// 0x024 FLTR register
} I2C ;

//#define I2C1	((I2C *)0x40005400)	// I2C1 base adress
#define I2C2	((I2C *)0x40005800)	// I2C2 base adress
#define I2C3	((I2C *)0x40005C00)	// I2C3 base adress

//===================  SYSCFG ==================
typedef struct
{
  volatile unsigned int MEMRMP;    // 0x000  memory remap register
  volatile unsigned int PMC;		// 0x004  peripheral mode configuration register
  volatile unsigned int EXTICR1;    // 0x008  external interrupt configuration register 1
  volatile unsigned int EXTICR2;	// 0x00C  external interrupt configuration register 2
  volatile unsigned int EXTICR3; 	// 0x010  external interrupt configuration register 3   
  volatile unsigned int EXTICR4;	// 0x014  external interrupt configuration register 1
  unsigned int reserved[2];
  volatile unsigned int CMPCR;   	 // 0x020  Compensation cell control register  
} _SYSCFG ;

#define SYSCFG	((_SYSCFG *)0x40013800) // SYSCFG base adress

//===================  EXTI ==================
typedef struct
{
  volatile unsigned int IMR;    // 0x000  Interrupt mask register
  volatile unsigned int EMR;	// 0x004  Event mask register
  volatile unsigned int RTSR;    // 0x008  Rising trigger selection register
  volatile unsigned int FTSR;	 // 0x00C Falling trigger selection register
  volatile unsigned int SWIER;    // 0x10 Software interrupt event register
  volatile unsigned int PR;       // 0x14 Pending register
} _EXTI ;

#define EXTI	((_EXTI *)0x40013C00) // EXTI base adress

//===================  NVIC ==================
typedef struct
{
  volatile unsigned int ISER[8];    // 0x000  Interrupt Set Enable Register 
  unsigned int RESERVED0[24];
  volatile unsigned int ICER[8];    // 0x080  Interrupt Clear Enable Register 
  unsigned int RSERVED1[24];
  volatile unsigned int ISPR[8];    // 0x100  Interrupt Set Pending Register 
  unsigned int RESERVED2[24];
  volatile unsigned int ICPR[8];    // 0x180  Interrupt Clear Pending Register 
  unsigned int RESERVED3[24];
  volatile unsigned int IABR[8];    // 0x200  Interrupt Active bit Register 
  unsigned int RESERVED4[56];
  volatile unsigned char  IPR[240];    // 0x300  Interrupt Priority Register (8Bit wide) 
  unsigned int RESERVED5[644];
  volatile  unsigned int STIR;      // 0xE00  Software Trigger Interrupt Register 
} STM32_NVIC ;

#define NVIC	((STM32_NVIC *) 0xe000e100) // NVIC base adress

//======================================
typedef struct systick {
	volatile unsigned int CSR;
	volatile unsigned int RVR;
	volatile unsigned int CVR;
	volatile unsigned int CAL;
} _SYSTIC;

#define SYSTICK 	((_SYSTIC *) 0xE000E010)

//======================   TIMERS ============
typedef struct {
	volatile unsigned int CR1;	    // 00  control register1 
	volatile unsigned int CR2;	    // 04  control register2 
	volatile unsigned int SMCR;	  // 08  slave mode control register 
	volatile unsigned int DIER;	  // 0c  DMA/Interrupt enable register 
	volatile unsigned int SR;	    // 10  status register 
	volatile unsigned int EGR;	    // 14  event generation register 
	volatile unsigned int CCMR[2];	// 18  capture/compare mode registers 1 @ 2
	volatile unsigned int CCER;	  // 20  capture/compare enable register 
	volatile unsigned int CNT;	    // 24  counter 
	volatile unsigned int PSC;	    // 28  prescaler 
	volatile unsigned int ARR;	    // 2c  auto-reload register 
	volatile unsigned int RCR;     // 30  repetition counter register
	volatile unsigned int CCR[4];	// 34  capture/compare registers 1 TO 4 
	volatile unsigned int BDTR;    // 44  break and dead-time register
	volatile unsigned int DCR;	    // 48  DMA control register 
	volatile unsigned int DMAR;	  // 4c  DMA address for full transfer 
} TIMER;

#define TIM1	((TIMER *) 0x40010000)  // TIMER1 base address
#define TIM2	((TIMER *) 0x40000000)
#define TIM3	((TIMER *) 0x40000400)
#define TIM4	((TIMER *) 0x40000800)

#define TIMER1_CC_IRQ 27  
#define	TIMER2_IRQn	28   
#define	TIM3_IRQn	29
#define	TIM4_IRQn	30

//======================  RCC ==============================
typedef struct rcc {
	volatile unsigned int CR;		//0 - control reg 
	volatile unsigned int PLLCFGR;		//4 - pll config 
	volatile unsigned int CFGR;		//8 - clock config 
	volatile unsigned int CIR;		//c - clock interrupt 
	volatile unsigned int AHB1RSTR;	//10 - AHB1 peripheral reset 
	volatile unsigned int AHB2RSTR;	//14 - AHB2 peripheral reset 
	int __pad1[2];
	volatile unsigned int APB1RSTR;	//20 - APB1 peripheral reset 
	volatile unsigned int APB2RSTR;	//24 - APB2 peripheral reset 
	int __pad2[2];
	volatile unsigned int AHB1ENR;	//30 - AHB1 peripheral enable 
	volatile unsigned int AHB2ENR;	//34 - AHB2 peripheral enable 
	int __pad3[2];
	volatile unsigned int APB1ENR;	//40 - APB1 peripheral enable 
	volatile unsigned int APB2ENR;	//44 - APB2 peripheral enable 
	int __pad4[2];
	volatile unsigned int AHB1LPENR;	//50 - AHB1 peripheral enable in low power 
	volatile unsigned int AHB2LPENR;	//54 - AHB2 peripheral enable in low power 
	int __pad5[2];
	volatile unsigned int APB1LPENR;	//60 - APB1 peripheral enable in low power 
	volatile unsigned int APB2LPENR;	//64 - APB2 peripheral enable in low power 
	int __pad6[2];
	volatile unsigned int BDCR;		//70 Backup domain control register
	volatile unsigned int CSR;		//74 clock control & status register
	int __pad7[2];
	volatile unsigned int SSCGR;	//80 spread spectrum clock generation register
	volatile unsigned int PLLI2SCFGR;	//84 PLLI2S configuration register
	int __pad8;
	volatile unsigned int DCKCFGR;		//8c Dedicated Clocks Configuration Register
} _RCC;

#define RCC	((_RCC *)0x40023800)  // 

//=======================
typedef struct gpio {
	volatile unsigned int MODER;	// 0x00 port mode register
	volatile unsigned int OTYPER;	// 0x04	port output type register
	volatile unsigned int OSPEEDR;	// 0x08 port output speed register
	volatile unsigned int PUPDR;	// 0x0c port pull-up/pull-down register
	volatile unsigned int IDR;		// 0x10 port input data register
	volatile unsigned int ODR;		// 0x14 port output data register
	volatile unsigned int BSRR;		// 0x18 port bit set/reset register
	volatile unsigned int LCKR;		// 0x1c port configuration lock register
	volatile unsigned int AFRL;		// 0x20 alternate function low register
	volatile unsigned int AFRH;		// 0x24 alternate function high register
} GPIO;

#define GPIOA ((GPIO *)0x40020000)
#define GPIOB ((GPIO *)0x40020400)
#define GPIOC ((GPIO *)0x40020800)


//=======================
typedef struct spi {
	volatile unsigned int cr1;	// 0x00
	volatile unsigned int cr2;	// 0x04
	volatile unsigned int sr;	// 0x08
	volatile unsigned int dr;	// 0x0c
	volatile unsigned int crcpr;	// 0x10
	volatile unsigned int rxcrcr;	// 0x14
	volatile unsigned int txcrcr;	// 0x18
	volatile unsigned int i2scgfr;	// 0x1c
	volatile unsigned int i2spr;	// 0x20
} SPI;

#define SPI1 ((SPI *)0x40013000)

//=======================
typedef struct dma {
	volatile unsigned int lisr;	// 0x00
	volatile unsigned int hisr;	// 0x04
	volatile unsigned int lifcr;	// 0x08
	volatile unsigned int hifcr;	// 0x0c
	volatile unsigned int s0[6];	// 0x10
	volatile unsigned int s1[6];	// 0x14
	volatile unsigned int s2[6];	// 0x18
	volatile unsigned int s3[6];	// 0x1c
	volatile unsigned int s4[6];	// 0x10
	volatile unsigned int s5[6];	// 0x14
	volatile unsigned int s6[6];	// 0x18
	volatile unsigned int s7[6];	// 0x1c
} DMA;

#define DMA2 ((DMA *)0x40026400)
// DMA2_Stream3 interrupt No 59
//=======================
typedef struct flash {
	volatile unsigned int ACR;		// 00 access control register
	volatile unsigned int KEYR;		// 04 key register
	volatile unsigned int OPTKEYR;	// 
	volatile unsigned int SR;
	volatile unsigned int CR;
	volatile unsigned int OPTCR;
} _FLASH;

#define FLASH ((_FLASH *)0x40023c00)


